#pragma once
#include <osg/Referenced>
#include <osgTerrain/GeometryTechnique>
#include <osg/Vec4d>
#include <osgTerrain/TerrainTile>


/**
 * \brief This class is the interface definition class for modifying terrain data. Must be subclassed to implement own terrain modification techniques.
 * 
 * @author Torben Dannhauer
 * @date  Feb 2011
 */ 
class terrainModificationTechnique : public osg::Referenced
{
public:
	terrainModificationTechnique();
	virtual ~terrainModificationTechnique();

	/**
	 * \brief This function performs the terrain data modification. Must be subclassed.
	 * 
	 * @param modificationROI This vector contains the extents of the region of interest which should be modified: lat_min, lat_max, long_min, long_max
	 * @param h : Heightfield of the tile which should be modified
	 * @param tileExtends : This vector contains the extents of the tile: lat_min, lat_max, long_min, long_max
	 */ 
	virtual void modifyHeightfield(osg::Vec4d& modificationROI, osg::HeightField* h, osg::Vec4d tileExtends)=0;

protected:
	void clampValue(double& value, double min, double max){if(value<min)value=min;if(value>max)value=max;}
	void clampValue(int& value, int min, int max){if(value<min)value=min;if(value>max)value=max;}
};